#ifndef __G31DDCAPI_H__
#define __G31DDCAPI_H__

#ifdef WIN32_LEAN_AND_MEAN

#include <basetyps.h>

#endif

#include <windows.h>

#define CLASS_ID_DEVICE_ENUMERATOR  0x1001
#define CLASS_ID_DEVICE             0x1002

#define INTERFACE_TYPE_PCIE    0
#define INTERFACE_TYPE_USB     1

#define INTERFACE_DEVICE_ENUMERATOR IG31DDCDeviceEnumerator
#define INTERFACE_DEVICE            IG31DDCDevice
#define INTERFACE_DEVICE_CALLBACK   IG31DDCDeviceCallback
#define STRUCT_DEVICE_INFO          G31DDC_DEVICE_INFO
#define STRUCT_DDC_TYPE_INFO        G31DDC_DDC_TYPE_INFO

#define FRONT_PANEL_LED_MODE_DIAG    0
#define FRONT_PANEL_LED_MODE_ON      1
#define FRONT_PANEL_LED_MODE_OFF     2

typedef BOOL (__stdcall *G31DDCAPI_CREATE_INSTANCE)(UINT32 ClassId,PVOID *Interface);

#pragma pack(push,1)


typedef struct
{
    char        DevicePath[MAX_PATH];
    UINT8       InterfaceType;
    char        SerialNumber[9];
    UINT32      DDCCount;
    UINT32      DDCTypeCount;
} STRUCT_DEVICE_INFO;

typedef struct
{
    UINT32      SampleRate;
    UINT32      Bandwidth;
    UINT32      BitsPerSample;
} STRUCT_DDC_TYPE_INFO;

#pragma pack(pop)

#undef INTERFACE
#define INTERFACE INTERFACE_DEVICE_ENUMERATOR

DECLARE_INTERFACE(INTERFACE_DEVICE_ENUMERATOR)
{
    STDMETHOD_(LONG,AddRef)(THIS) PURE;
    STDMETHOD_(LONG,Release)(THIS) PURE;
    STDMETHOD_(BOOL,Enumerate)(THIS) PURE;
    STDMETHOD_(UINT32,GetCount)(THIS) PURE;
    STDMETHOD_(BOOL,GetDeviceInfo)(THIS_ UINT32 DeviceIndex,STRUCT_DEVICE_INFO *DeviceInfo,UINT32 BufferLength) PURE;
};

#undef INTERFACE
#define INTERFACE INTERFACE_DEVICE_CALLBACK

struct INTERFACE_DEVICE;

DECLARE_INTERFACE(INTERFACE_DEVICE_CALLBACK)
{
    STDMETHOD_(VOID,IFStreamCallback)(struct INTERFACE_DEVICE *Device,CONST INT16 *Buffer,UINT32 NumberOfSamples,UINT16 Amplitude,UINT16 ADCSamplingRate) PURE;
    STDMETHOD_(VOID,DDCStreamCallback)(struct INTERFACE_DEVICE *Device,CONST VOID *Buffer,UINT32 NumberOfSamples,UINT32 BitsPerSample) PURE;
};

#undef INTERFACE
#define INTERFACE INTERFACE_DEVICE


DECLARE_INTERFACE(INTERFACE_DEVICE)
{
    STDMETHOD_(LONG,AddRef)(THIS) PURE;
    STDMETHOD_(LONG,Release)(THIS) PURE;

    STDMETHOD_(BOOL,Open)(THIS_ CONST CHAR *DevicePath) PURE;
    STDMETHOD_(VOID,Close)(THIS) PURE;
    STDMETHOD_(BOOL,IsOpen)(THIS) PURE;

    STDMETHOD_(BOOL,IsConnected)(THIS) PURE;

    STDMETHOD_(VOID,SetCallback)(THIS_ INTERFACE_DEVICE_CALLBACK *Callback) PURE;
    STDMETHOD_(INTERFACE_DEVICE_CALLBACK*,GetCallback)(THIS) PURE;

    STDMETHOD_(BOOL,GetDeviceInfo)(THIS_ STRUCT_DEVICE_INFO *Info,UINT32 BufferLength) PURE;

    STDMETHOD_(BOOL,SetPower)(THIS_ BOOL Power) PURE;
    STDMETHOD_(BOOL,GetPower)(THIS_ BOOL *Power) PURE;

    STDMETHOD_(BOOL,StartIF)(THIS_ UINT16 Period) PURE;
    STDMETHOD_(BOOL,StopIF)(THIS) PURE;

    STDMETHOD_(BOOL,GetDDCTypeInfo)(THIS_ UINT32 DDCTypeIndex,STRUCT_DDC_TYPE_INFO *Info) PURE;
    STDMETHOD_(BOOL,SetDDCType)(THIS_ UINT32 DDCTypeIndex) PURE;
    STDMETHOD_(BOOL,GetDDCType)(THIS_ UINT32 *DDCTypeIndex) PURE;
    STDMETHOD_(BOOL,StartDDC)(THIS_ UINT32 SamplesPerBuffer) PURE;
    STDMETHOD_(BOOL,StopDDC)(THIS) PURE;
    STDMETHOD_(BOOL,SetDDCFrequency)(THIS_ UINT32 Frequency) PURE;
    STDMETHOD_(BOOL,GetDDCFrequency)(THIS_ UINT32 *Frequency) PURE;

    STDMETHOD_(BOOL,SetAttenuator)(THIS_ UINT32 Attenuator) PURE;
    STDMETHOD_(BOOL,GetAttenuator)(THIS_ UINT32 *Attenuator) PURE;

    STDMETHOD_(BOOL,SetDithering)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetDithering)(THIS_ BOOL *Enabled) PURE;

    STDMETHOD_(BOOL,SetMWFilter)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetMWFilter)(THIS_ BOOL *Enabled) PURE;

    STDMETHOD_(BOOL,SetLED)(THIS_ UINT32 LEDMode) PURE;
    STDMETHOD_(BOOL,GetLED)(THIS_ UINT32 *LEDMode) PURE;

    STDMETHOD_(BOOL,SetInverted)(THIS_ BOOL Inverted) PURE;
    STDMETHOD_(BOOL,GetInverted)(THIS_ BOOL *Inverted) PURE;

    STDMETHOD_(BOOL,GetSpectrumCompenstation)(THIS_ UINT64 CenterFrequency,UINT32 Width,float *Buffer,UINT32 Count) PURE;
};

#endif


